#include "kanji_union.h"

extern UTF8_TABLE *u;
KANJI kanji_union;
KANJI *double_byte_character = &kanji_union;

static int ERROR_CODE = 1;

int inverted(char *c) {
    char *p = (char *)double_byte_character->sequence;

// Copying in reverse order
    p[0] = c[1];
    p[1] = c[0];
    return double_byte_character->code[0];
}

int reading_to_produce_UTF8(REGULATE *r, char *new_name) {
    BOOL *which = & r->which_table;
    BOOL kanji = FALSE;
    char *p;
    char a[3] = {0, 0, 0};
    int c, k, m;
    int exit_code = 0;
    FILE *file;
    unsigned char uc;
    unsigned char first_byte = 0x40;
    FILE *new_file = fopen(new_name, "wb");
    if(new_file == NULL) {
        new_file = fopen(SJIS_UTF8_TEXT, "wb");
    }

    if(! setting(u, r->no_searching, which) ) {
        error_handler("reading_to_produce_UTF8() error:", "setting(u, r->no_searching, which)");
        fclose(new_file);
        return ERROR_CODE;
    }

    file = fopen(r->document_name, "rt");
    if(file == NULL) {
        fprintf(stderr, "fopen() error-1: %s\n", r->document_name);
        fclose(new_file);
        return ERROR_CODE;
    }

    for(;;) {
        p = fgets(r->buffer, BUFFER_SIZE, file);
        if(p == NULL) {
            break;
        } else {
            p = strchr(r->buffer, '\n');
            if(p != NULL) {
                *p = '\0';
            }

            k = 0;
        }

        for(m = 0; r->buffer[m]; m++) {
            uc = (unsigned char)r->buffer[m];
            c = (int)uc;

            if(kanji) {
                a[0] = (char)first_byte;
                a[1] = (char)uc;
                r->combo[k] = working(u, inverted(a), 0);
                r->combo[ k + 1 ] = working(u, inverted(a), 1);
                r->combo[ k + 2 ] = working(u, inverted(a), 2);
                k += UTF8_KANJI_SIZE;
                kanji = FALSE;
                continue;
            }

            if(0x81 <= c && c<= 0xFC) {
                if(KANA_KUTEN <= c && c <= KANA_HANDAKUTEN) {
                    r->combo[k] = working(u, c, 0);
                    r->combo[ k + 1 ] = working(u, c, 1);
                    r->combo[ k + 2 ] = working(u, c, 2);
                    k += UTF8_KANJI_SIZE;
                    kanji = FALSE;
                } else {
                    first_byte = uc;
                    kanji = TRUE;
                }

            } else {
                r->combo[k++] = (char)uc;
                kanji = FALSE;
            }

        }

        r->combo[k++] = 0x0A;
        fwrite(r->combo, k, 1, new_file);
    }

    fclose(file);
    fclose(new_file);
    detach_table(u);
    release_buffers(r);
    return exit_code;
}

int reading_to_produce_UTF8_CRLF(REGULATE *r, char *new_name) {
    BOOL *which = & r->which_table;
    BOOL kanji = FALSE;
    char *p;
    char a[3] = {0, 0, 0};
    int c, k, m;
    int exit_code = 0;
    FILE *file;
    unsigned char uc;
    unsigned char first_byte = 0x40;
    FILE *new_file = fopen(new_name, "wb");
    if(new_file == NULL) {
        new_file = fopen(SJIS_UTF8_TEXT, "wb");
    }

    if(! setting(u, r->no_searching, which) ) {
        error_handler("reading_to_produce_UTF8() error:", "setting(u, r->no_searching, which)");
        fclose(new_file);
        return ERROR_CODE;
    }

    file = fopen(r->document_name, "rt");
    if(file == NULL) {
        fprintf(stderr, "fopen() error-3: %s\n", r->document_name);
        fclose(new_file);
        return ERROR_CODE;
    }

    for(;;) {
        p = fgets(r->buffer, BUFFER_SIZE, file);
        if(p == NULL) {
            break;
        } else {
            p = strchr(r->buffer, '\n');
            if(p != NULL) {
                *p = '\0';
            }

            k = 0;
        }

        for(m = 0; r->buffer[m]; m++) {
            uc = (unsigned char)r->buffer[m];
            c = (int)uc;

            if(kanji) {
                a[0] = (char)first_byte;
                a[1] = (char)uc;
                r->combo[k] = working(u, inverted(a), 0);
                r->combo[ k + 1 ] = working(u, inverted(a), 1);
                r->combo[ k + 2 ] = working(u, inverted(a), 2);
                k += UTF8_KANJI_SIZE;
                kanji = FALSE;
                continue;
            }

            if(0x81 <= c && c<= 0xFC) {
                if(KANA_KUTEN <= c && c <= KANA_HANDAKUTEN) {
                    r->combo[k] = working(u, c, 0);
                    r->combo[ k + 1 ] = working(u, c, 1);
                    r->combo[ k + 2 ] = working(u, c, 2);
                    k += UTF8_KANJI_SIZE;
                    kanji = FALSE;
                } else {
                    first_byte = uc;
                    kanji = TRUE;
                }

            } else {
                r->combo[k++] = (char)uc;
                kanji = FALSE;
            }

        }

        r->combo[k++] = 0x0D;
        r->combo[k++] = 0x0A;
        fwrite(r->combo, k, 1, new_file);
    }

    fclose(file);
    fclose(new_file);
    detach_table(u);
    release_buffers(r);
    return exit_code;
}

BOOL allocating(REGULATE *r, char *d, BOOL n, BOOL w) {
    BOOL allocated = TRUE;

    strcpy(r->document_name, d);
    r->no_searching = n;
    r->which_table = w;
    r->buffer = malloc(BUFFER_SIZE);
    if(r->buffer == NULL) {
        fprintf(stderr, "malloc() error-5: %d bytes", BUFFER_SIZE);
        allocated = FALSE;
    }

    r->combo = malloc(BUFFER_SIZE);
    if(r->combo == NULL) {
        fprintf(stderr, "r->combo == NULL error: %d bytes", BUFFER_SIZE);
        allocated = FALSE;
    }

    return allocated;
}

int produce_temporary_binary_file(const char *name, BOOL no_removing) {
    char c[BLOCK_SIZE];
    FILE *ancient = fopen(name, "rb");
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "wb");
    if(temporary == NULL) {
        fprintf(stderr, "fopen() error: %s\n", TEMPORARY_BINARY);
        exit(1);
    }

    if(ancient == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, ancient);
            fwrite(c, size, 1, temporary);
        } while(size == BLOCK_SIZE);

        fclose(temporary);
        fclose(ancient);

        if(! no_removing) {
            remove(name);
        }

    }

    return exit_code;
}

int rename_binary_file(const char *name, BOOL no_removing) {
    char c[BLOCK_SIZE];
    int exit_code = 0;
    int size;
    FILE *temporary = fopen(TEMPORARY_BINARY, "rb");
    FILE *recent = fopen(name, "wb");
    if(recent == NULL) {
        recent = fopen(SJIS_UTF8_TEXT, "wb");
    }

    if(temporary == NULL) {
        exit_code = FILE_ERROR_CODE;
    } else {
        do {
            size = fread(c, 1, BLOCK_SIZE, temporary);
            fwrite(c, size, 1, recent);
        } while(size == BLOCK_SIZE);

        fclose(recent);
        fclose(temporary);

        if(! no_removing) {
            remove(TEMPORARY_BINARY);
        }

    }

    return exit_code;
}

void error_handler(const char *p, const char *q) {
    FILE *file = fopen(ERROR_TEXT, "wt");
    fprintf(file, "%s %s", p, q);;
    fclose(file);
}

void release_buffers(REGULATE *r) {
    free(r->buffer);
    free(r->combo);
}
