#include "kanji_union.h"

/*
sjisUTF8 C-1.07
Converting the text from Shift_JIS to UTF-8
Copyright 2019 Sogaya

gcc (GCC) 3.4.2 (mingw-special)
Borland C++ 5.5.1 for Win32
Open Watcom C32 Optimizing Compiler Version 1.8 J001
Digital Mars Compiler Version 8.42n (Digital Mars C/C++ Compiler Version 8.57)

sjisUTF8 assumes that the little-endian architecture underlies operating systems (Linux, Windows).
*/

extern UTF8_TABLE *u;

int main(int argc, char *argv[]) {
    const BOOL REMOVE_ANCIENT_FILE = FALSE;
    const BOOL WITHOUT_COMMENTS = FALSE;
    const BOOL WITH_COMMENTS = TRUE;
    const char SETTING_TEXT[] = "set8.txt";
    const int ERROR_CODE = 1;
    char DIFFERENT_TIME[] = "-t-";
    char DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE[] = "-w-";
    char NO_SKIPPING[] = "-s-";
    char SEARCH_TABLE[] = "-d-";
    char VERSION_WITH_COMMENTS[] = "-VC";
    char VERSION_WITHOUT_COMMENTS[] = "-V";

    BOOL carriage_return = FALSE;
    BOOL identical_time = FALSE;
    BOOL no_removing = FALSE;
    BOOL no_searching = FALSE;
    BOOL produce_binary_file = FALSE;
    BOOL skip_control_characters = FALSE;
    BOOL file_to_be_read = TRUE;
    BOOL which_table = FALSE;
    char* binary_name = u->binary_name;
    char sjis_text[ FILE_NAME_SIZE + 1 ];
    char utf8_text[ FILE_NAME_SIZE + 1 ];
    int file_name_size;
    int exit_code = 0;
    int i;
    REGULATE regulate;
    REGULATE *r = &regulate;

    memset(sjis_text, 0, FILE_NAME_SIZE + 1 );

    binary_name[FILE_NAME_SIZE] = '\0';
    strncpy(binary_name, SJIS_UTF8_BIN, FILE_NAME_SIZE);

    utf8_text[FILE_NAME_SIZE] = '\0';
    strncpy(utf8_text, SJIS_UTF8_TEXT, FILE_NAME_SIZE);

/* Acquiring parameters from SETTING_TEXT unless the user removes it. */
    acquire_parameters_from_text(SETTING_TEXT, binary_name, sjis_text, utf8_text,  &produce_binary_file, &carriage_return, &no_searching, &skip_control_characters, &identical_time, &which_table, &no_removing);

    for(i = 1; i < argc; ++i) {
        if(! strcmp(CARRIAGE_RETURN_LINE_FEED, argv[i]) ) {
            carriage_return = TRUE;
        } else if(! strcmp(DIFFERENT_TIME, argv[i]) ) {
            identical_time = FALSE;
        } else if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, argv[i]) ) {
            no_removing = TRUE;
        } else if(! strcmp(DO_NOT_SEARCH_TABLE, argv[i]) ) {
            no_searching = TRUE;
        } else if(! strcmp(DO_NOT_SHOW_WHICH_TABLE_IS_ACCESSIBLE, argv[i]) ) {
            which_table = FALSE;
        } else if(! strcmp(IDENTICAL_TIME, argv[i]) ) {
            identical_time = TRUE;
        } else if(! strcmp(LINE_FEED, argv[i]) ) {
            carriage_return = FALSE;
        } else if(! strcmp(NO_SKIPPING, argv[i]) ) {
            skip_control_characters = FALSE;
        } else if(! strcmp(SEARCH_TABLE, argv[i]) ) {
            no_searching = FALSE;
        } else if(! strcmp(SKIPPING, argv[i]) ) {
            skip_control_characters = TRUE;
        } else if(! strcmp(TEMPORARY_BINARY_FILE, argv[i]) ) {
            produce_binary_file = TRUE;
        } else if(! strcmp(VERSION_WITH_COMMENTS, argv[i]) ) {
            version_guide(VERSION, WITH_COMMENTS);
        } else if(! strcmp(VERSION_WITHOUT_COMMENTS, argv[i]) ) {
            version_guide(VERSION, WITHOUT_COMMENTS);
        } else if(! strcmp(WHICH_TABLE, argv[i]) ) {
            which_table = TRUE;
        } else {
            if(file_to_be_read) {
                sjis_text[FILE_NAME_SIZE] = '\0';
                strncpy(sjis_text, argv[i], FILE_NAME_SIZE);
                file_to_be_read = FALSE;
            } else {
                utf8_text[FILE_NAME_SIZE] = '\0';
                strncpy(utf8_text, argv[i], FILE_NAME_SIZE);
            }

        }

    }

    if(! allocating(r, sjis_text, no_searching, which_table) ) {
        return ERROR_CODE;
    }

    file_name_size = strlen(sjis_text);
    if(file_name_size == 0) {
        if(skip_control_characters) {
            exit_code = standard_input_output_skipping(r);
        } else {
            exit_code = standard_input_output(r);
        }

    } else {
        if(identical_time) {
            produce_temporary_binary_file(sjis_text, REMOVE_ANCIENT_FILE);
            rename_binary_file(sjis_text, no_removing);
        }

        if(produce_binary_file) {
            if(carriage_return) {
                if(skip_control_characters) {
                    exit_code = skipping_control_characters_to_produce_UTF8_CRLF(r, TEMPORARY_BINARY);
                } else {
                    exit_code = reading_to_produce_UTF8_CRLF(r, TEMPORARY_BINARY);
                }

            } else {
                if(skip_control_characters) {
                    exit_code = skipping_control_characters_to_produce_UTF8(r, TEMPORARY_BINARY);
                } else {
                    exit_code = reading_to_produce_UTF8(r, TEMPORARY_BINARY);
                }

            }

            rename_binary_file(utf8_text, no_removing);
        } else {
            if(carriage_return) {
                if(skip_control_characters) {
                    exit_code = skipping_control_characters_to_produce_UTF8_CRLF(r, utf8_text);
                } else {
                    exit_code = reading_to_produce_UTF8_CRLF(r, utf8_text);
                }

            } else {
                if(skip_control_characters) {
                    exit_code = skipping_control_characters_to_produce_UTF8(r, utf8_text);
                } else {
                    exit_code = reading_to_produce_UTF8(r, utf8_text);
                }

            }

        }

    }

    return exit_code;
}

/*
1:  binary_name                       MS932~UTF-8.bin
2:  document_name                     KeyWords.csv
3:  new_name                          sjisUTF8.txt
4:  TEMPORARY_BINARY_FILE             -b
5:  CARRIAGE_RETURN_LINE_FEED         -CRLF
6:  DO_NOT_SEARCH_TABLE               -d
7:  SKIPPING                          -s
8:  IDENTICAL_TIME                    -t
9:  WHICH_TABLE                       -w
10: DO_NOT_REMOVE_TEMPORARY_BINARY    -y
*/

void acquire_parameters_from_text(const char *parameters_text, char *binary_name, char *document_name, char *new_name, BOOL *produce_binary_file, BOOL *carriage_return, BOOL *no_searching, BOOL *skip_control_characters, BOOL *identical_time, BOOL *which_table, BOOL *no_removing) {
    char *p;
    char c[C_SIZE];
    FILE *setting = fopen(parameters_text, "rt");
    int k;
    int line_number = 0;

    if(setting != NULL) {
        for(;;) {
            p = fgets(c, C_SIZE, setting);
            if(p == NULL) {
                break;
            } else {
                k = strcspn(c, "\n',|\t");
                c[k] = '\0';
            }

            line_number += 1;
            switch(line_number) {
            case 1:
                strcpy(binary_name, c); /* MS932~UTF-8.bin */
                break;
            case 2:
                strcpy(document_name, c); /* KeyWords.csv */
                break;
            case 3:
                strcpy(new_name, c); /* sjisUTF8.txt */
                break;
            case 4:
                if(! strcmp(TEMPORARY_BINARY_FILE, c) ) { /* -b */
                    *produce_binary_file = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *produce_binary_file = FALSE;
                } else {
                    *produce_binary_file = TRUE;
                }

                break;
            case 5:
                if(! strcmp(CARRIAGE_RETURN_LINE_FEED, c) ) { /* -CRLF */
                    *carriage_return = TRUE;
                } else if(! strcmp(LINE_FEED, c) ) { /* -LF */
                    *carriage_return = FALSE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *carriage_return = FALSE;
                } else {
                    *carriage_return = TRUE;
                }

                break;
            case 6:
                if(! strcmp(DO_NOT_SEARCH_TABLE, c) ) { /* -d */
                    *no_searching = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *no_searching = FALSE;
                } else {
                    *no_searching = TRUE;
                }

                break;
            case 7:
                if(! strcmp(SKIPPING, c) ) { /* -s */
                    *skip_control_characters = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *skip_control_characters = FALSE;
                } else {
                    *skip_control_characters = TRUE;
                }

                break;
            case 8:
                if(! strcmp(IDENTICAL_TIME, c) ) { /* -t */
                    *identical_time = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *identical_time = FALSE;
                } else {
                    *identical_time = TRUE;
                }

                break;
            case 9:
                if(! strcmp(WHICH_TABLE, c) ) { /* -w */
                    *which_table = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *which_table = FALSE;
                } else {
                    *which_table = TRUE;
                }

                break;
            case 10:
                if(! strcmp(DO_NOT_REMOVE_TEMPORARY_BINARY_FILE, c) ) { /* -y */
                    *no_removing = TRUE;
                } else if(! strcmp(ZERO, c) ) { /* 0 */
                    *no_removing = FALSE;
                } else {
                    *no_removing = TRUE;
                }

                break;
            default:
                break;
            } /* switch(line_number) */

        } /* for(;;) */

        fclose(setting);
    } /* if(setting != NULL) */

}
